module Klastera
  class ApplicationController < ::ApplicationController
    protect_from_forgery with: :exception
    layout :disable_in_xhr_requests
    before_action :set_xhr_render_session
    before_action :check_access_to_cluster_admin

    def check_access_to_cluster_admin
      unless cluster_user.can_admin_clusters?
        redirect_to main_app.root_path, flash: { alert: t('klastera.messages.access_cluster_admin') }
      end
    end

    if respond_to?(:skip_after_action)
      skip_after_action :verify_authorized
    end

    private
    def disable_in_xhr_requests
      request.xhr? ? false : nil
    end

    def set_xhr_render_session
      return unless request.get?
      # request.xhr? return nil when this is not a XHR request, but we need a boolean
      session[:xhr_render] = request.xhr?.present?
    end
  end
end