module Klastera::Concerns::ClusterUser
  extend ActiveSupport::Concern

  included do
    self.table_name = 'cluster_users'
    belongs_to :user
    belongs_to :cluster
    validates :cluster_id, presence: true
  end
  
  module ClassMethods
    ##
     # Return a Cluster::ActiveRecord_Relation of organization (and) user
     #
    def clusters_of(organization,and_user=nil)
      and_user_id = and_user.present? ? { users: { id: and_user } }  : {}
      ::Cluster.eager_load(cluster_users: :user).where({ organization_id: organization }.merge(and_user_id) ).order(order: :asc)
    end

    ##
     # Return a User::ActiveRecord_Relation of organization (and) user
     #
    def users_of(organization)
      ::User.eager_load(:cluster_users).where(users: { organization_id: organization } )
    end

    ##
     # Return a hash of users and its clusters
     #
    def users_hash_of(organization)
      users = {}
      rows = self.users_of(organization).pluck("users.id AS user_id","cluster_users.cluster_id").uniq
      rows.each do |row|
        user_id = row.first
        users[user_id] ||= []
        users[user_id] << row.last
      end
      users
    end
  end
end