module Klastera::Concerns::Clusterizable
  extend ActiveSupport::Concern

  included do
    class MutipleClustersOperationError < StandardError; end

    belongs_to :cluster

    has_many :cluster_entities, as: :entity, class_name: "Klastera::ClusterEntity"
    accepts_nested_attributes_for :cluster_entities, reject_if: :all_blank, allow_destroy: true

    validates :cluster_id, presence: true, if: -> { cluster_id.present? && organization.present? && organization.required_suborganization_mode? }
    validate :at_least_one_cluster_entity, if: -> { organization.present? && organization.required_suborganization_mode? }
    validate :uniqueness_of_cluster_entity_record

    scope :includes_cluster, -> { includes(cluster_entities: :cluster) }

    def at_least_one_cluster_entity
      if cluster_entities.length == 0 || cluster_entities.reject{|cluster_entity| cluster_entity._destroy == true}.empty?
        return errors.add(:cluster_entities, I18n.t('klastera.messages.at_least_one_cluster_entity'))
      end
    end

    def allow_multiple_clusters?(default=:one)
      organization.present? && organization.cluster_cardinality_of(self.class, default: default) == :many
    end

    ##
     # This is a legacy method and we don't recommend using it.
     # Implement directly Klastera.entity_clusters_string_list instead of this method.
     # TODO: In order to deprecate it, you will need to perform some changes in the main app
     ##
    def clusters_string_separated_by(separator,attribute=:name)
      Klastera.entity_clusters_string_list!(
        cluster_entities, separator, attribute
      )
    end

    def has_one_cluster_entity
      if cluster_entities.length > 1
        return errors.add(:cluster_entities, I18n.t('klastera.messages.has_one_cluster_entity'))
      end
    end

    def uniqueness_of_cluster_entity_record
      if cluster_entities.map(&:cluster_id).uniq.size != cluster_entities.size
        return errors.add(:cluster_entities, I18n.t('klastera.messages.duplicated_cluster_entity') )
      end
    end
  end
  
  module ClassMethods
    def cluster_entity_params
      [ :cluster_id, { cluster_entities_attributes: [:id, :cluster_id, :_destroy] } ]
    end
  end
end